@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003 Christian Loose <christian.loose@hamburg.de>
+ * Copyright (c) 2002-2004 Christian Loose <christian.loose@kdemail.net>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,8 +20,6 @@
 
 #include "cvsservice.h"
 
-#include <memory>
-
 #include <qintdict.h>
 #include <qstring.h>
 
@@ -54,14 +52,15 @@ struct CvsService::Private
         delete singleCvsJob;
     }
 
-    CvsJob*             singleCvsJob;   // non-concurrent cvs job, like update or commit
-    DCOPRef             singleJobRef;   // DCOP reference to non-concurrent cvs job
-    QIntDict<CvsJob>    cvsJobs;        // concurrent cvs jobs, like diff or annotate
-    unsigned            lastJobId;
+    CvsJob*               singleCvsJob;   // non-concurrent cvs job, like update or commit
+    DCOPRef               singleJobRef;   // DCOP reference to non-concurrent cvs job
+    QIntDict<CvsJob>      cvsJobs;        // concurrent cvs jobs, like diff or annotate
+    QIntDict<CvsLoginJob> loginJobs;
+    unsigned              lastJobId;
 
-    QCString            appId;          // cache the DCOP clients app id
+    QCString              appId;          // cache the DCOP clients app id
 
-    Repository*         repository;
+    Repository*           repository;
 
     CvsJob* createCvsJob();
     DCOPRef setupNonConcurrentJob(Repository* repo = 0);
@@ -85,6 +84,7 @@ CvsService::CvsService()
     d->repository = new Repository();
 
     d->cvsJobs.setAutoDelete(true);
+    d->loginJobs.setAutoDelete(true);
 
     KConfig* config = kapp->config();
     KConfigGroupSaver cs(config, "General");
@@ -106,6 +106,7 @@ CvsService::~CvsService()
     ssh.killSshAgent();
 
     d->cvsJobs.clear();
+    d->loginJobs.clear();
     delete d;
 }
 
@@ -186,19 +187,20 @@ DCOPRef CvsService::annotate(const QString& fileName, const QString& revision)
 
 
 DCOPRef CvsService::checkout(const QString& workingDir, const QString& repository,
-                             const QString& module, const QString& tag, bool pruneDirs)
+                             const QString& module, const QString& tag, 
+                             bool pruneDirs)
 {
     if( d->hasRunningJob() )
         return DCOPRef();
 
-    std::auto_ptr<Repository> repo(new Repository(repository));
+    Repository repo(repository);
 
     // assemble the command line
     // cd [DIRECTORY] && cvs -d [REPOSITORY] checkout [-r tag] [-P] [MODULE]
     d->singleCvsJob->clearCvsCommand();
 
     *d->singleCvsJob << "cd" << KProcess::quote(workingDir) << "&&"
-                     << repo->cvsClient()
+                     << repo.cvsClient()
                      << "-d" << repository
                      << "checkout";
 
@@ -210,7 +212,43 @@ DCOPRef CvsService::checkout(const QString& workingDir, const QString& repositor
 
     *d->singleCvsJob << module;
 
-    return d->setupNonConcurrentJob(repo.get());
+    return d->setupNonConcurrentJob(&repo);
+}
+
+
+DCOPRef CvsService::checkout(const QString& workingDir, const QString& repository,
+                             const QString& module, const QString& tag, 
+                             bool pruneDirs, const QString& alias, bool exportOnly)
+{
+    if( d->hasRunningJob() )
+        return DCOPRef();
+
+    Repository repo(repository);
+
+    // assemble the command line
+    // cd [DIRECTORY] && cvs -d [REPOSITORY] co [-r tag] [-P] [-d alias] [MODULE]
+    d->singleCvsJob->clearCvsCommand();
+
+    *d->singleCvsJob << "cd" << KProcess::quote(workingDir) << "&&"
+                     << repo.cvsClient()
+                     << "-d" << repository;
+    if( exportOnly)
+      *d->singleCvsJob << "export";
+    else
+      *d->singleCvsJob << "checkout";
+
+    if( !tag.isEmpty() )
+        *d->singleCvsJob << "-r" << tag;
+
+    if( pruneDirs && !exportOnly )
+        *d->singleCvsJob << "-P";
+
+    if( !alias.isEmpty() )
+      *d->singleCvsJob << "-d" << alias;
+
+    *d->singleCvsJob << module;
+
+    return d->setupNonConcurrentJob(&repo);
 }
 
 
@@ -236,6 +274,24 @@ DCOPRef CvsService::commit(const QStringList& files, const QString& commitMessag
 }
 
 
+DCOPRef CvsService::createRepository(const QString& repository)
+{
+    if( d->hasRunningJob() )
+        return DCOPRef();
+        
+    // assemble the command line
+    // cvs -d [REPOSITORY] init
+    d->singleCvsJob->clearCvsCommand();
+    
+    *d->singleCvsJob << "mkdir -p" << KProcess::quote(repository) << "&&"
+                     << d->repository->cvsClient() 
+                     << "-d" << KProcess::quote(repository)
+                     << "init";
+
+    return d->setupNonConcurrentJob();
+}
+
+
 DCOPRef CvsService::createTag(const QStringList& files, const QString& tag,
                               bool branch, bool force)
 {
@@ -286,6 +342,25 @@ DCOPRef CvsService::deleteTag(const QStringList& files, const QString& tag,
 }
 
 
+DCOPRef CvsService::downloadCvsIgnoreFile(const QString& repository,
+                                          const QString& outputFile)
+{
+    Repository repo(repository);
+
+    // create a cvs job
+    CvsJob* job = d->createCvsJob();
+
+    // assemble the command line
+    // cvs -d [REPOSITORY] -q checkout -p CVSROOT/cvsignore > [OUTPUTFILE]
+    *job << repo.cvsClient() << "-d" << repository 
+         << "-q checkout -p CVSROOT/cvsignore >" 
+         << KProcess::quote(outputFile);
+
+    // return a DCOP reference to the cvs job
+    return DCOPRef(d->appId, job->objId());
+}
+
+
 DCOPRef CvsService::downloadRevision(const QString& fileName,
                                      const QString& revision,
                                      const QString& outputFile)
@@ -340,6 +415,16 @@ DCOPRef CvsService::downloadRevision(const QString& fileName,
 DCOPRef CvsService::diff(const QString& fileName, const QString& revA,
                          const QString& revB, const QString& diffOptions,
                          unsigned contextLines)
+{
+    // cvs diff [DIFFOPTIONS] -U CONTEXTLINES [-r REVA] {-r REVB] [FILE]
+    QString format = "-U" + QString::number(contextLines);
+    return diff(fileName, revA, revB, diffOptions, format);
+}
+
+
+DCOPRef CvsService::diff(const QString& fileName, const QString& revA,
+                         const QString& revB, const QString& diffOptions,
+                         const QString& format)
 {
     if( !d->hasWorkingCopy() )
         return DCOPRef();
@@ -348,9 +433,9 @@ DCOPRef CvsService::diff(const QString& fileName, const QString& revA,
     CvsJob* job = d->createCvsJob();
 
     // assemble the command line
-    // cvs diff [DIFFOPTIONS] -U CONTEXTLINES [-r REVA] {-r REVB] [FILE]
+    // cvs diff [DIFFOPTIONS] [FORMAT] [-r REVA] {-r REVB] [FILE]
     *job << d->repository->cvsClient() << "diff" << diffOptions
-         << "-U" << QString::number(contextLines);
+         << format;
 
     if( !revA.isEmpty() )
         *job << "-r" << KProcess::quote(revA);
@@ -422,18 +507,58 @@ DCOPRef CvsService::import(const QString& workingDir, const QString& repository,
     if( d->hasRunningJob() )
         return DCOPRef();
 
-    std::auto_ptr<Repository> repo(new Repository(repository));
+    Repository repo(repository);
 
     // assemble the command line
     d->singleCvsJob->clearCvsCommand();
 
-    *d->singleCvsJob << "cd" << workingDir << "&&"
-                     << repo->cvsClient()
+    *d->singleCvsJob << "cd" << KProcess::quote(workingDir) << "&&"
+                     << repo.cvsClient()
                      << "-d" << repository
                      << "import";
 
     if( importAsBinary )
         *d->singleCvsJob << "-kb";
+        
+    const QString ignore = ignoreList.stripWhiteSpace();
+    if( !ignore.isEmpty() )
+        *d->singleCvsJob << "-I" << KProcess::quote(ignore);
+
+    QString logMessage = comment.stripWhiteSpace();
+    logMessage.prepend("\"");
+    logMessage.append("\"");
+    *d->singleCvsJob << "-m" << logMessage;
+
+    *d->singleCvsJob << module << vendorTag << releaseTag;
+
+    return d->setupNonConcurrentJob(&repo);
+}
+
+
+DCOPRef CvsService::import(const QString& workingDir, const QString& repository,
+                           const QString& module, const QString& ignoreList,
+                           const QString& comment, const QString& vendorTag,
+                           const QString& releaseTag, bool importAsBinary,
+                           bool useModificationTime)
+{
+    if( d->hasRunningJob() )
+        return DCOPRef();
+
+    Repository repo(repository);
+
+    // assemble the command line
+    d->singleCvsJob->clearCvsCommand();
+
+    *d->singleCvsJob << "cd" << KProcess::quote(workingDir) << "&&"
+                     << repo.cvsClient()
+                     << "-d" << repository
+                     << "import";
+
+    if( importAsBinary )
+        *d->singleCvsJob << "-kb";
+        
+    if( useModificationTime )
+        *d->singleCvsJob << "-d";
 
     const QString ignore = ignoreList.stripWhiteSpace();
     if( !ignore.isEmpty() )
@@ -446,7 +571,7 @@ DCOPRef CvsService::import(const QString& workingDir, const QString& repository,
 
     *d->singleCvsJob << module << vendorTag << releaseTag;
 
-    return d->setupNonConcurrentJob(repo.get());
+    return d->setupNonConcurrentJob(&repo);
 }
 
 
@@ -488,21 +613,20 @@ DCOPRef CvsService::login(const QString& repository)
     if( repository.isEmpty() )
         return DCOPRef();
 
-    std::auto_ptr<Repository> repo(new Repository(repository));
+    Repository repo(repository);
 
     // create a cvs job
     ++(d->lastJobId);
 
-    // TODO: we need to delete this job object
     CvsLoginJob* job = new CvsLoginJob(d->lastJobId);
+    d->loginJobs.insert(d->lastJobId, job);
 
-    // TODO: CVS_RSH and CVS_SERVER don't work ATM
-//    job->setRSH(repo->rsh());
-//    job->setServer(repo->server());
+    // TODO: CVS_SERVER doesn't work ATM
+//    job->setServer(repo.server());
 
     // assemble the command line
     // cvs -d [REPOSITORY] login
-    job->setCvsClient(repo->clientOnly().local8Bit());
+    job->setCvsClient(repo.clientOnly().local8Bit());
     job->setRepository(repository.local8Bit());
 
     // return a DCOP reference to the cvs job
@@ -515,7 +639,7 @@ DCOPRef CvsService::logout(const QString& repository)
     if( repository.isEmpty() )
         return DCOPRef();
 
-    std::auto_ptr<Repository> repo(new Repository(repository));
+    Repository repo(repository);
 
     // create a cvs job
     ++(d->lastJobId);
@@ -523,13 +647,13 @@ DCOPRef CvsService::logout(const QString& repository)
     CvsJob* job = new CvsJob(d->lastJobId);
     d->cvsJobs.insert(d->lastJobId, job);
 
-    job->setRSH(repo->rsh());
-    job->setServer(repo->server());
-    job->setDirectory(repo->workingCopy());
+    job->setRSH(repo.rsh());
+    job->setServer(repo.server());
+    job->setDirectory(repo.workingCopy());
 
     // assemble the command line
     // cvs -d [REPOSITORY] logout
-    *job << repo->cvsClient() << "-d" << repository << "logout";
+    *job << repo.cvsClient() << "-d" << repository << "logout";
 
     // return a DCOP reference to the cvs job
     return DCOPRef(d->appId, job->objId());
@@ -537,6 +661,12 @@ DCOPRef CvsService::logout(const QString& repository)
 
 
 DCOPRef CvsService::makePatch()
+{
+    return makePatch("", "-u");
+}
+
+
+DCOPRef CvsService::makePatch(const QString& diffOptions, const QString& format)
 {
     if( !d->hasWorkingCopy() )
         return DCOPRef();
@@ -545,8 +675,8 @@ DCOPRef CvsService::makePatch()
     CvsJob* job = d->createCvsJob();
 
     // assemble the command line
-    // cvs diff -uR 2>/dev/null
-    *job << d->repository->cvsClient() << "diff" << "-uR"
+    // cvs diff [DIFFOPTIONS] [FORMAT] -R 2>/dev/null
+    *job << d->repository->cvsClient() << "diff" << diffOptions << format << "-R"
          << "2>/dev/null";
 
     // return a DCOP reference to the cvs job
@@ -556,7 +686,7 @@ DCOPRef CvsService::makePatch()
 
 DCOPRef CvsService::moduleList(const QString& repository)
 {
-    std::auto_ptr<Repository> repo(new Repository(repository));
+    Repository repo(repository);
 
     // create a cvs job
     ++(d->lastJobId);
@@ -564,13 +694,13 @@ DCOPRef CvsService::moduleList(const QString& repository)
     CvsJob* job = new CvsJob(d->lastJobId);
     d->cvsJobs.insert(d->lastJobId, job);
 
-    job->setRSH(repo->rsh());
-    job->setServer(repo->server());
-    job->setDirectory(repo->workingCopy());
+    job->setRSH(repo.rsh());
+    job->setServer(repo.server());
+    job->setDirectory(repo.workingCopy());
 
     // assemble the command line
     // cvs -d [REPOSITORY] checkout -c
-    *job << repo->cvsClient() << "-d" << repository << "checkout -c";
+    *job << repo.cvsClient() << "-d" << repository << "checkout -c";
 
     // return a DCOP reference to the cvs job
     return DCOPRef(d->appId, job->objId());
