@@ -3,6 +3,8 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
+		2001	  by Stanislav Visnovsky
+			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,12 +18,20 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 **************************************************************************** */
 #ifndef FINDOPTIONS_H
 #define FINDOPTIONS_H
 
+#include <qregexp.h>
+
+class QString;
+class QChar;
+
+namespace KBabel 
+{
+
 class FindOptions
 {
 public:
@@ -39,6 +49,18 @@ public:
 
    bool ignoreAccelMarker;
    bool ignoreContextInfo;
+   
+   bool askForNextFile;
+   bool askForSave;	// whether should ask before saving or save automatically when loading next file
+   bool inAllFiles;	// whether search should be done in all files
+   bool inMarkedFiles;  // whether search should be restricted to marked ones
+   bool inTemplates;
+
+   // these are non-persistent options - not saved into configuration database
+   bool askFile;	// whether findNext should ask catalog manager for next file
+   QRegExp contextInfo;	// actual settings of context info regular expression
+   QChar accelMarker;	// actual settings of accelerator marker
+   
 };
 
 class ReplaceOptions : public FindOptions
@@ -48,5 +70,6 @@ public:
    bool ask;
 };
 
+}
 
 #endif
