@@ -16,18 +16,16 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 **************************************************************************** */
 
 #include "kbcatalog.h"
-#include "kbabelview.h"
 #include "headereditor.h"
-#include "editcmd.h"
 
 
-KBCatalog::KBCatalog(QObject* parent, const char* name) 
-         : Catalog(parent,name)
+KBCatalog::KBCatalog(QString configFile, QObject* parent, const char* name) 
+         : KBabel::Catalog(parent,name,configFile)
 {
    _headerEditor=0;
 }
