@@ -561,6 +561,7 @@ MsgMultiLineEdit::MsgMultiLineEdit(int ID, KSpell* spell, QWidget* parent,const
                 _diffAddColor(Qt::darkGreen),
                 _diffDelColor(Qt::darkRed),
 		_currentUnicodeNumber(0),
+		highlighter(0),
 		_tagStartPara(0), _tagStartIndex(0), _tagEndPara(0), _tagEndIndex(0)
 {
    diffPos.setAutoDelete(true);
@@ -587,12 +588,19 @@ MsgMultiLineEdit::MsgMultiLineEdit(int ID, KSpell* spell, QWidget* parent,const
    // this will setup bitBlt pixmaps
    setFont( font() );
    highlighter = new KBabelHighlighter( this, spell );
+   connect( this, SIGNAL( signalSyntaxHighlightingChanged( bool ) ), highlighter, SLOT( setSyntaxHighlighting( bool ) ) );
    
    connect( this, SIGNAL( selectionChanged() ), this, SLOT( paintSpacePoints() ) );   
    connect( this, SIGNAL( cursorPositionChanged( int, int ) ), this, SLOT( paintSpacePoints(int, int) ) );
    connect( this, SIGNAL( textChanged() ), this, SLOT( emittedTextChanged() ) );
 }
 
+MsgMultiLineEdit::~MsgMultiLineEdit ()
+{
+    if(highlighter)
+	delete highlighter;
+}
+
 void MsgMultiLineEdit::setText(const QString& s)
 {
     QString str = s;
@@ -791,6 +799,8 @@ void MsgMultiLineEdit::setHighlightSyntax(bool on)
 {
    _hlSyntax=on;
    
+   emit signalSyntaxHighlightingChanged (on);
+   
    update();
 }
 
