@@ -16,7 +16,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 **************************************************************************** */
 
@@ -35,7 +35,7 @@ public:
 	PcFactory( QObject *parent=0, const char *name=0);
 	~PcFactory();
 
-	virtual QObject *create( QObject *parent=0, const char *name=0
+	virtual QObject *createObject( QObject *parent=0, const char *name=0
 					, const char *classname="QObject"
 					, const QStringList &args = QStringList());
 
