@@ -16,13 +16,14 @@
 
 #include "umlclassifierlist.h"
 #include "umlassociationlist.h"
-#include "dialogs/settingsdlg.h"
+#include "optionstate.h"
 #include "umlviewlist.h"
 #include "umlobjectlist.h"
 #include "umlinterfacelist.h"
 #include "umldatatypelist.h"
 #include "umlnamespace.h"
 #include "umlattributelist.h"
+#include "umlstereotypelist.h"
 #include <qdatastream.h>
 #include <qmap.h>
 #include <qdict.h>
@@ -44,15 +45,16 @@ class QSplitter;
 
 class KPrinter;
 
+class ClassImport;
 class CodeGenerator;
 class DocWindow;
 class IDChangeLog;
 class ObjectWidget;
-class UMLListView;
 class UMLView;
 class UMLWidget;
 class UMLClass;
 class UMLClassifier;
+class UMLOperation;
 class UMLPackage;
 class UMLEnum;
 class UMLApp;
@@ -81,7 +83,7 @@ public:
 	/**
 	 * Constructor for the fileclass of the application
 	 */
-	UMLDoc(QWidget *parent, const char *name=0);
+	UMLDoc();
 
 	/**
 	 * Destructor for the fileclass of the application
@@ -101,7 +103,7 @@ public:
 	 *
 	 * @param view		Pointer to the UMLView to remove.
 	 * @param enforceOneView	switch to determine if we have a current view or not.
-         *                          most of the time, we DO want this, except when exiting the program.
+	 *                          most of the time, we DO want this, except when exiting the program.
 	 */
 	void removeView(UMLView *view , bool enforceOneView = true );
 
@@ -189,14 +191,6 @@ public:
 	 */
 	void setupSignals();
 
-	/**
-	 * Gains a link to the list view so that it can set the signals
-	 * up required by this class.
-	 *
-	 * @param lv		Pointer to the UMLListView.
-	 */
-	void setupListView(UMLListView *lv);
-
 	/**
 	 * Returns true if the given name is unique within its scope.
 	 *
@@ -205,6 +199,14 @@ public:
 	 */
 	bool isUnique(QString name);
 
+       /**
+         * Returns true if the given name is unique within its scope of given package.
+         *
+         * @param name          The name to check.
+         * @return      True if name is unique.
+         */
+	bool isUnique(QString name, UMLPackage *package); 
+
 	/**
 	 * Creates a @ref UMLObject of the given type.
 	 *
@@ -273,17 +275,14 @@ public:
 	 * Creates an enum literal for the parent enum.
 	 *
 	 * @param o	The parent enum
-	 * @return	The UMLEmun created
+	 * @return	The UMLEnum created
 	 */
 	UMLObject* createEnumLiteral(UMLEnum* umlenum);
 
 	/**
-	 * Creates a stereotype for the parent classifier.
-	 *
-	 * @param o	The parent concept
-	 * @return	The UMLStereotype created
+	 * Finds or creates a stereotype for the parent object.
 	 */
-	UMLObject* createStereotype(UMLClassifier* classifier, UMLObject_Type list);
+	UMLStereotype* UMLDoc::findOrCreateStereotype(QString name);
 
 	/**
 	 * Creates an operation for the parent concept.
@@ -329,7 +328,7 @@ public:
 	 * @param visibilityB	The visibility of role B (optional)
 	 */
 	/*
-         void addAssociation(QString name, Association_Type assocType,
+	void addAssociation(QString name, Association_Type assocType,
 			int AId, int BId,
 			QString nameA = "", QString nameB = "",
 			QString multiA = "", QString multiB = "",
@@ -343,7 +342,7 @@ public:
 	 *
 	 * @param pAssoc	Pointer to the UMLAssociation to add.
 	 */
-         void addAssociation(UMLAssociation *pAssoc);
+	void addAssociation(UMLAssociation *pAssoc);
 
 	/**
 	 * Removes an association.
@@ -365,8 +364,8 @@ public:
 	 * @return	Pointer to the UMLAssociation found or NULL if not found.
 	 */
 	UMLAssociation * findAssociation(Uml::Association_Type assocType,
-					 UMLObject *roleAObj,
-					 UMLObject *roleBObj,
+					 const UMLObject *roleAObj,
+					 const UMLObject *roleBObj,
 					 bool *swap = NULL);
 
 	/**
@@ -378,14 +377,14 @@ public:
 	 */
 	void createDiagram(Diagram_Type type, bool askForName = true);
 
-        /**
-         * Removes an @ref UMLObject from the current file.  If this object
-         * is being represented on a diagram it will also delete all those
-         * representations.
-         *
-         * @param o		Pointer to the UMLObject to delete.
-         */
-        void removeUMLObject(UMLObject*o);
+	/**
+	 * Removes an @ref UMLObject from the current file.  If this object
+	 * is being represented on a diagram it will also delete all those
+	 * representations.
+	 *
+	 * @param o		Pointer to the UMLObject to delete.
+	 */
+	void removeUMLObject(UMLObject*o);
 
 
 	/**
@@ -475,6 +474,14 @@ public:
 	 */
 	UMLClassifier * findUMLClassifier (QString name);
 
+	/**
+	 * Finds a UMLStereotype by its name.
+	 *
+	 * @param name		The name of the UMLStereotype to find.
+	 * @return	Pointer to the UMLStereotype found, or NULL if not found.
+	 */
+	UMLStereotype * findStereotype(QString name);
+
 	/**
 	 * Finds a view (diagram) by the ID given to method.
 	 *
@@ -519,7 +526,7 @@ public:
 	 *
 	 * @param file		The file to be saved to.
 	 */
-	virtual bool saveToXMI(QIODevice& file);
+	virtual void saveToXMI(QIODevice& file);
 
 	/**
 	 * Checks the given XMI file if it was saved with correct Unicode
@@ -554,6 +561,13 @@ public:
 	 */
 	bool loadUMLObjectsFromXMI( QDomElement & element );
 
+	/**
+	 * Loads umbrello specific extensions from XMI to the UMLDoc.
+	 * The extension tags are: <docsettings>, <diagrams>, <listview>,
+	 * and <codegeneration>.
+	 */
+	void loadExtensionsFromXMI(QDomNode & node);
+
 	/**
 	 * Loads all diagrams from XMI into the current UMLDoc.
 	 *
@@ -611,38 +625,35 @@ public:
 	 * @return	Pointer to the current UMLView.
 	 */
 	UMLView * getCurrentView() {
-		return currentView;
-	}
-
-	/**
-	 * Returns the list view.
-	 *
-	 * @return	Pointer to the current UMLListView.
-	 */
-	UMLListView * getListView() {
-		return listView;
+		return m_currentView;
 	}
 
 	/**
 	 * Returns a list of the concepts in this UMLDoc.
 	 *
+	 * @param includeNested		Whether to include the concepts from
+	 *				nested packages (default: true.)
 	 * @return	List of UML concepts.
 	 */
-	UMLClassifierList getConcepts();
+	UMLClassifierList getConcepts(bool includeNested = true);
 
 	/**
 	 * Returns a list of the classes and interfaces in this UMLDoc.
 	 *
+	 * @param includeNested		Whether to include the concepts from
+	 *				nested packages (default: true.)
 	 * @return	List of UML concepts.
 	 */
-	UMLClassifierList getClassesAndInterfaces();
+	UMLClassifierList getClassesAndInterfaces(bool includeNested = true);
 
 	/**
 	 * Returns a list of the interfaces in this UMLDoc.
 	 *
+	 * @param includeNested		Whether to include the concepts from
+	 *				nested packages (default: true.)
 	 * @return	List of UML interfaces.
 	 */
-	UMLInterfaceList getInterfaces();
+	UMLInterfaceList getInterfaces(bool includeNested = true);
 
 	/**
 	 * Returns a list of the datatypes in this UMLDoc.
@@ -675,16 +686,14 @@ public:
 	}
 
 	/**
-	 * Adds an already created UMLObject to the document, it gets
-	 * assigned a new ID, if its name is already in use then the
-	 * function appends a number to it to differentiate it from
-	 * the others; this number is incremental so if number 1 is in
-	 * use then it tries 2 and then 3 and so on
+	 * Assigns an already created UMLObject a new ID.
+	 * If the object is a classifier then the operations/attributes 
+	 * are also assigned new IDs.
 	 *
 	 * @param Obj		Pointer to the UMLObject to add.
 	 * @return	True if operation successful.
 	 */
-	bool addUMLObjectPaste(UMLObject* Obj);
+	bool assignNewIDs(UMLObject* Obj);
 
 	/**
 	 * Adds an already created UMLView to the document, it gets
@@ -725,18 +734,6 @@ public:
 	 */
 	int assignNewID(int OldID);
 
-	/**
-	 * Returns the document window.
-	 *
-	 * @return	Pointer to the DocWindow.
-	 */
-	DocWindow * getDocWindow();
-
-	/** a friendly little utility method to allow accessing
-	 * the parent UMLApps option state.
-	 */
-	SettingsDlg::OptionState getOptionState();
-
 	/**
 	 * Returns the documentation for the project.
 	 *
@@ -755,14 +752,6 @@ public:
 		m_Doc = doc;
 	}
 
-	/**
-	 * Activate a diagram after loading a new document
-	 *
-	 * @param viewID	ID of the UMLView to activate.
-	 * @return	True on successful activation.
-	 */
-	bool activateView ( int viewID );
-
 	/**
 	 * Activate all the diagrams/views after loading so all their
 	 * widgets keep their IDs.
@@ -771,43 +760,10 @@ public:
 	 */
 	bool activateAllViews();
 
-	/**
-	 * Passes onto @ref UMLApp that an event equivalent to
-	 * edit copy has occured.
-	 */
-	void editCopy();
-
-	/**
-	 * Passes onto @ref UMLApp that an event
-	 * equivalent to edit cut has occured.
-	 */
-	void editCut();
-
-	/**
-	 * Passes onto @ref UMLApp that an event equivalent to
-	 * edit paste has occured.
-	 */
-	void editPaste();
-
-	/**
-	 * Returns the paste state
-	 */
-	bool getPasteState();
-
-	/**
-	 * Returns the state on Cut/Copy
-	 */
-	bool getCutCopyState();
-
-	/**
-	 * Returns the central dock widget used for diagrams
-	 */
-	QWidget* getMainViewWidget();
-
 	/**
 	 * Sets the default settings to the given settings.
 	 */
-	void settingsChanged(SettingsDlg::OptionState optionState);
+	void settingsChanged(Settings::OptionState optionState);
 
 
 	/**
@@ -860,11 +816,6 @@ public:
 	 */
 	UMLViewList m_ViewList;
 
-	/**
-	 * The tree view of diagrams and objects.
-	 */
-	UMLListView* listView;
-
 	/**
 	 * Returns a name for the new object, appended with a number
 	 * if the default name is taken e.g. class diagram, class
@@ -879,21 +830,26 @@ public:
 		return m_bLoading;
 	}
 
-        /**
-         * Find a code generator by the given language.
-         */
-        CodeGenerator * findCodeGeneratorByLanguage (QString lang);
+	/**
+	 * Return the ClassImport object.
+	 */
+	ClassImport * classImport();
 
 	/**
-         * Add a CodeGenerator object to this UMLDoc
+	 * Find a code generator by the given language.
+	 */
+	CodeGenerator * findCodeGeneratorByLanguage (QString lang);
+
+	/**
+	 * Add a CodeGenerator object to this UMLDoc
 	 */
 	bool addCodeGenerator ( CodeGenerator * add_gen );
 
-        /**
-         * Remove and delete a CodeGenerator object from this UMLDoc.
-         * @return boolean - will return false if it couldnt remove a generator.
-         */
-        bool removeCodeGenerator ( CodeGenerator * remove_object );
+	/**
+	 * Remove and delete a CodeGenerator object from this UMLDoc.
+	 * @return boolean - will return false if it couldnt remove a generator.
+	 */
+	bool removeCodeGenerator ( CodeGenerator * remove_object );
 
 	/** Set the current (active) code generator for this document.
 	 */
@@ -914,6 +870,42 @@ public:
 	 */
 	void createDatatype(QString name);
 
+	/**
+	 * Add a UMLObject to the application.
+	 */
+	void addObject(UMLObject *o);
+
+	/**
+	 * Find a UMLStereotype by its unique ID.
+	 */
+	UMLStereotype * findStereotype(int id);
+
+	/**
+	 * Add a UMLStereotype to the application.
+	 */
+	void addStereotype(const UMLStereotype *s);
+
+	/**
+	 * Remove a UMLStereotype from the application.
+	 */
+	void removeStereotype(const UMLStereotype *s);
+
+	/**
+	 * Returns a name for the new object, appended with a number
+	 * if the default name is taken e.g. new_actor, new_actor_1
+	 * etc.
+	 * @param type		The object type.
+	 * @param prefix	The prefix to use (optional.)
+	 *			If no prefix is given then a type related
+	 *			prefix will be chosen internally.
+	 */
+	QString uniqObjectName(const UMLObject_Type type, QString prefix = "");
+
+	/**
+	 * Write text to the status bar.
+	 */
+	void writeToStatusBar(const QString &text);
+
 private:
 
 	/**
@@ -922,13 +914,6 @@ private:
 	 */
 	void removeAssocFromConcepts(UMLAssociation *assoc);
 
-	/**
-	 * Returns a name for the new object, appended with a number
-	 * if the default name is taken e.g. new actor, new actor_1
-	 * etc.
-	 */
-	QString uniqObjectName(const UMLObject_Type type);
-
 	/**
 	 * Utility method for the public findUMLObject() method: Find the
 	 * UML object of the given type and name in the passed-in list.
@@ -957,36 +942,44 @@ private:
 	 */
 	bool objectTypeIsClassifierListItem(UMLObject_Type type);
 
+	ClassImport * m_classImporter;
 	CodeGenerator * m_currentcodegenerator;
-	UMLObjectList objectList;
-	int uniqueID;
+	UMLObjectList m_objectList;
+
+	/**
+	 * The UMLDoc is the sole owner of all stereotypes.
+	 * UMLStereotype instances are reference counted.
+	 * When an UMLStereotype is no longer referenced anywhere,
+	 * its refcount drops to zero. It is then removed from the
+	 * m_stereoList and it is physically deleted.
+	 */
+	UMLStereotypeList m_stereoList;
+
+	int m_uniqueID;
 	int m_count;   ///< auxiliary counter for the progress bar
 	bool m_modified;
-	KURL doc_url;
-	UMLView* currentView;
+	KURL m_doc_url;
+	UMLView* m_currentView;
+	/// Auxiliary variable for loading foreign XMI files
+	int m_highestIDforForeignFile;
 
 	/**
 	 * A dictionary of the parameters in the save XMI file
 	 * sorted by language.
 	 */
-        QMap<QString, QDomElement> * m_codeGenerationXMIParamMap;
+	QMap<QString, QDomElement> * m_codeGenerationXMIParamMap;
 
-        /**
+	/**
 	 * A dictionary of various code generators we currently have
 	 * configured for this UML document.
 	 */
-        QDict<CodeGenerator> m_codeGeneratorDictionary;
+	QDict<CodeGenerator> m_codeGeneratorDictionary;
 
 	/**
 	 * Contains all the UMLObject id changes of paste session.
 	 */
 	IDChangeLog* m_pChangeLog;
 
-	/**
-	 * List of Not yet Activated views.
-	 */
-	UMLViewList viewsNotActivated;
-
 	/**
 	 * true if the we're loading a new document
 	 */
@@ -1021,17 +1014,21 @@ private:
 	 */
 	QPtrList<QDataStream> redoStack;
 
+	/**
+	 * Auxiliary to <docsettings> processing
+	 */
+	int m_nViewID;
 
 public slots:
 
-       /**
-         * Adds a UMLObject thats already created but doesn't change
-         * any ids or signal.  Used by the list view.  Use
-         * AddUMLObjectPaste if pasting.
-         */
-        void addUMLObject( UMLObject * object );
+	/**
+	 * Adds a UMLObject thats already created but doesn't change
+	 * any ids or signal.  Used by the list view.  Use
+	 * AddUMLObjectPaste if pasting.
+	 */
+	void addUMLObject( UMLObject * object );
 
-        void slotRemoveUMLObject(UMLObject*o);
+	void slotRemoveUMLObject(UMLObject*o);
 
 
 	/**
@@ -1063,7 +1060,7 @@ signals:
 	 */
 	void sigResetStatusbarProgress();
 
-	/*
+	/**
 	 * Set the total range of the progressbar.
 	 *
 	 * @param totalSteps Total range of the progressbar (0..totalSteps)
@@ -1071,7 +1068,7 @@ signals:
 	void sigSetStatusbarProgressSteps(int totalSteps);
 
 
-	/*
+	/**
 	 * Set the progress position of the progressbar.
 	 *
 	 * @param stepPosition	The step position to set.
