@@ -14,35 +14,32 @@
 #include <config.h>
 #endif
 
-#include "umlobjectlist.h"
-#include "umlwidgetlist.h"
-#include "messagewidgetlist.h"
-#include "associationwidgetlist.h"
-
-#include "umlnamespace.h"
-#include "worktoolbar.h"
-#include "floatingtext.h"
-#include "dialogs/settingsdlg.h"
-
-// QT includes
+// system includes
+#include <kurl.h>
+#include <qdom.h>
 #include <qcanvas.h>
 
-// KDE includes
-#include <kurl.h>
-
-// #define OFFSET		50
+//local includes
+#include "umlobjectlist.h"
+#include "umlwidgetlist.h"
+#include "associationwidgetlist.h"
+#include "messagewidgetlist.h"
+#include "optionstate.h"
+#include "worktoolbar.h"
 
+// forward declarations
 class ClassOptionsPage;
 class IDChangeLog;
 class ListPopupMenu;
+class FloatingText;
 class SeqLineWidget;
 class ObjectWidget;
 
-class UMLListView;
 class UMLApp;
 class UMLDoc;
 
 class KPrinter;
+class ToolBarStateFactory;
 
 using namespace Uml;
 
@@ -58,18 +55,23 @@ using namespace Uml;
  */
 class UMLView : public QCanvasView {
 	Q_OBJECT
+
+	friend class ToolBarState;
+	friend class ToolBarStateOther; // TODO
+	friend class ToolBarStateAssociation; // TODO
+	friend class ToolBarStateMessages; // TODO
 public:
 	/**
 	 * Constructor for the main view
 	 */
-	UMLView(QWidget* parent, UMLDoc* doc);
+	UMLView();
 
 	/**
 	 * Destructor for the main view
 	 */
 	~UMLView();
 
-        // Accessors and other methods dealing with loaded/saved data
+	// Accessors and other methods dealing with loaded/saved data
 
 	/**
 	 * Return the documentation of the diagram.
@@ -134,6 +136,13 @@ public:
 		return m_Options.uiState.lineColor;
 	}
 
+	/**
+	 * Returns the line width to use.
+	 */
+	uint getLineWidth() const {
+		return m_Options.uiState.lineWidth;
+	}
+
 	/**
 	 * Sets the line color.
 	 *
@@ -141,6 +150,13 @@ public:
 	 */
 	void setLineColor( QColor color );
 
+	/**
+	 * Sets the line width.
+	 *
+	 * @param width		The width to use.
+	 */
+	void setLineWidth( uint width );
+
 	/**
 	 * Returns the ID of the diagram.
 	 */
@@ -285,14 +301,14 @@ public:
 	/**
 	 * Returns the options being used.
 	 */
-	const SettingsDlg::OptionState& getOptionState() const {
+	const Settings::OptionState& getOptionState() const {
 		return m_Options;
 	}
 
 	/**
 	 * Sets the options to be used.
 	 */
-	void setOptionState( const SettingsDlg::OptionState& options) {
+	void setOptionState( const Settings::OptionState& options) {
 		m_Options = options;
 	}
 
@@ -317,7 +333,7 @@ public:
 		return m_MessageList;
 	}
 
-        // End of accessors and methods that only deal with loaded/saved data
+	// End of accessors and methods that only deal with loaded/saved data
 	////////////////////////////////////////////////////////////////////////
 
 	/**
@@ -325,17 +341,6 @@ public:
 	*/
 	int currentZoom();
 
-	/**
-	 * returns a pointer to the document connected to the view
-	 * instance. Mind that this method requires a UMLApp instance
-	 * as a parent widget to get to the window document pointer by
-	 * calling the UMLApp::getDocument() method.
-	 *
-	 * @see UMLApp#getDocument
-	 */
-	UMLDoc* getDocument() const;
-
-
 	/**
 	 * contains the implementation for printing functionality
 	 */
@@ -365,13 +370,6 @@ public:
 	 */
 	void showEvent(QShowEvent *se);
 
-	/**
-	 * Returns a reference to the @ref UMLView class.
-	 *
-	 * @return Returns a reference to the @ref UMLView class.
-	 */
-	UMLListView * getListView();
-
 	/**
 	 * Sets an association to include the given widget.  If this is the second
 	 * widget set for the association it creates the association.
@@ -470,6 +468,11 @@ public:
 	 */
 	void selectionSetLineColor( QColor color );
 
+	/**
+	 * Set the line width for all the currently selected items.
+	 */
+	void selectionSetLineWidth( uint width );
+
 	/**
 	 * Set the fill color for all the currently selected items.
 	 */
@@ -810,7 +813,7 @@ public:
 	/**
 	 * Creates the <diagram> tag and fills it with the contents of the diagram.
 	 */
-	virtual bool saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
+	virtual void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
 	/**
 	 * Loads the <diagram> tag.
@@ -822,6 +825,23 @@ public:
 	 */
 	UMLWidget* loadWidgetFromXMI(QDomElement& widgetElement);
 
+	/**
+	 * Add an object to the application, and update the view.
+	 */
+	void addObject(UMLObject *object);
+
+	/**
+	 * Selects all the widgets within an internally kept rectangle.
+	 */
+	void selectWidgets(int px, int py, int qx, int qy);
+
+	/**
+	 * Determine whether on a sequence diagram we have clicked on a line of an Object.
+	 *
+	 * @returns	Returns the widget thats line was clicked on.  Returns 0 if no line was clicked on.
+	 */
+	ObjectWidget * onWidgetLine( QPoint point );
+
 protected:
 
 	// Methods and members related to loading/saving
@@ -861,7 +881,7 @@ protected:
 	/**
 	 * Options used by view
 	 */
-	SettingsDlg::OptionState	m_Options;
+	Settings::OptionState	m_Options;
 
 	/**
 	 * Contains all the data items for @ref MessageWidgets on the diagram.
@@ -937,18 +957,12 @@ protected:
 	 */
 	void contentsDropEvent(QDropEvent* mouseEvent);
 
-	/**
-	 * Determine whether on a sequence diagram we have clicked on a line of an Object.
-	 *
-	 * @returns	Returns the widget thats line was clicked on.  Returns 0 if no line was clicked on.
-	 */
-	ObjectWidget * onWidgetLine( QPoint point );
 
 	/**
 	 * Adds an AssociationWidget to the association list
 	 * and creates the corresponding UMLAssociation in the current UMLDoc.
 	 */
-        void addAssocInViewAndDoc(AssociationWidget* assoc);
+	void addAssocInViewAndDoc(AssociationWidget* assoc);
 
 	/**
 	 * Initializes key variables.
@@ -959,7 +973,7 @@ protected:
 	 * Initialize and announce a newly created widget.
 	 * Auxiliary to contentsMouseReleaseEvent().
 	 */
-	void setupNewWidget(UMLWidget *w, bool setNewID=true);
+	void setupNewWidget(UMLWidget *w);
 
 	/**
 	 * Overrides the standard operation.
@@ -988,10 +1002,6 @@ protected:
 	 */
 	QRect getDiagramRect();
 
-	/**
-	 * Selects all the widgets within an internally kept rectangle.
-	 */
-	void selectWidgets();
 
 	/**
 	 * Selects all the widgets of the given association widget.
@@ -1000,10 +1010,10 @@ protected:
 
 	/**
 	 * Calls setSelected on the given UMLWidget and enters
-         * it into the m_SelectedList while making sure it is
-         * there only once.
+	 * it into the m_SelectedList while making sure it is
+	 * there only once.
 	 */
-        void makeSelected (UMLWidget * uw);
+	void makeSelected (UMLWidget * uw);
 
 	/**
 	 * Updates the size of all components in this view.
@@ -1048,6 +1058,10 @@ public:
 	WorkToolBar::ToolBar_Buttons getCurrentCursor() const;
 
 private:
+
+	ToolBarStateFactory* m_pToolBarStateFactory;
+	ToolBarState* m_pToolBarState;
+
 	WorkToolBar::ToolBar_Buttons m_CurrentCursor;
 
 	/**
@@ -1055,31 +1069,6 @@ private:
 	 */
 	Uml::Association_Type convert_TBB_AT(WorkToolBar::ToolBar_Buttons tbb);
 
-	/**
-	 * convert toolbar button enums to umlobject type enums
-	 */
-	Uml::UMLObject_Type convert_TBB_OT(WorkToolBar::ToolBar_Buttons tbb);
-
-	/**
-	 * Sees if the MousePressEvent needs to be allocated to a UMLWidget
-	 */
-	bool allocateMousePressEvent(QMouseEvent * me);
-
-	/**
-	 * Sees if the MouseReleaseEvent needs to be allocated to a UMLWidget
-	 */
-	bool allocateMouseReleaseEvent(QMouseEvent * me);
-
-	/**
-	 * Sees if the MouseDoubleClickEvent needs to be allocated to a UMLWidget
-	 */
-	bool allocateMouseDoubleClickEvent(QMouseEvent * me);
-
-	/**
-	 * Sees if the MouseMoveEvent needs to be allocated to a UMLWidget
-	 */
-	bool allocateMouseMoveEvent(QMouseEvent * me);
-
 	/**
 	 * LocalID Changes Log for paste actions
 	 */
@@ -1196,7 +1185,8 @@ signals:
 	void sigColorChanged( int );
 	void sigRemovePopupMenu();
 	void sigClearAllSelected();
-	void sigLineColorChanged( QColor );
+	void sigLineColorChanged( int );
+	void sigLineWidthChanged( int );
 	void sigSnapToGridToggled(bool);
 	void sigSnapComponentSizeToGridToggled(bool);
 	void sigShowGridToggled(bool);
